#include <Arduino.h>
#include <DallasTemperature.h>
#include <OneWire.h>
#include <Adafruit_NeoPixel.h>
#include "controller.h"


/******************************************************************************
 *                                     DEFINES
 ******************************************************************************/

#define DELAY_TIME            1000 /**< time between measurements [ms] */
#define MIN_FAN_SPEED_PERCENT 24 /**< minimum fan speed [%] */
#define MIN_TEMP              25 /**< turn fan off below [deg C] */
#define MAX_TEMP              40 /**< turn fan to full speed above [deg C] */
#define INVALID_TEMP          (MAX_TEMP+MAX_TEMP) /**< Invalid temperature (on sensor failures)*/

#define TEMP_SENSOR_MEASURE_SERIES  5 /**< Maximum resets */

/******************************************************************************
 *                            LOCAL VARIABLES
 ******************************************************************************/
Adafruit_NeoPixel*  pPixelFan = NULL;
Adafruit_NeoPixel*  pPixel    = NULL;

OneWire oneWire(GPIO_DS18B20);
DallasTemperature sensors(&oneWire);

/**
 * @brief Called once at start
 * Initialize hardware pins
 */
void setup() {

  Serial.begin(115200);
  /* Setup FAN Control */
  pinMode(FAN_PIN, OUTPUT);
  pinMode(SIGNAL_PIN, INPUT);


  pPixelFan = new Adafruit_NeoPixel(WS2812_LEDS1, GPIO_WS2812_PIN1, NEO_RGB + NEO_KHZ800);

  pPixelFan->begin();
  pPixelFan->clear();
  pPixelFan->fill(pPixelFan->Color(255,0,0));
  pPixelFan->setBrightness(100);
  pPixelFan->show();

/* Initialize Temperature sensor */
  sensors.begin();
  for(int j=0; j < TEMP_SENSOR_MEASURE_SERIES && sensors.getDeviceCount() == 0; j++)
  {
    delay(100);
    sensors.begin();
    Serial.println("Reset 1-Wire Bus");
  }
}

/**
 * @brief Get the Fan Speed in round per minutes
 * 
 * @return int 
 */
int getFanSpeedRpm() {
  int highTime = pulseIn(SIGNAL_PIN, HIGH);
  int lowTime = pulseIn(SIGNAL_PIN, LOW);
  int period = highTime + lowTime;
  if (period == 0) {
    return 0;
  }
  float freq = 1000000.0 / (float)period;
  return (freq * 60.0) / 2.0; // two cycles per revolution
}

/**
 * @brief Set the Fan Speed
 * 
 * @param p expected percentage
 */
void setFanSpeedPercent(int p) {
  int value = (p / 100.0) * 255;
  analogWrite(FAN_PIN, value);
}

float readTemperature(void)
{
  int sensorCount = sensors.getDeviceCount();
  if (sensorCount > 0)
  {
    sensors.requestTemperatures();
    float temp1Raw = sensors.getTempCByIndex(0);
    if (temp1Raw > 0)
    {
      return temp1Raw;
    }
  }

  return INVALID_TEMP;
}

/**
 * @brief Endless loop
 * (is called as fast as possible)
 */
void loop() {
  float temp = readTemperature();  
  Serial.print(temp);
  Serial.print(";");

  int fanSpeedPercent, actualFanSpeedRpm;

  if (temp < MIN_TEMP) {
    fanSpeedPercent = 0;
  } else if (temp > MAX_TEMP) {
    fanSpeedPercent = 100;
  } else {
    fanSpeedPercent = (100 - MIN_FAN_SPEED_PERCENT) * (temp - MIN_TEMP) / (MAX_TEMP - MIN_TEMP) + MIN_FAN_SPEED_PERCENT;
  }

  pPixelFan->fill(pPixelFan->Color(0,255,0));
  pPixelFan->setBrightness(fanSpeedPercent);
  pPixelFan->show();

  Serial.print(fanSpeedPercent);
  Serial.print("%;");
  setFanSpeedPercent(fanSpeedPercent);

  actualFanSpeedRpm = getFanSpeedRpm();
  Serial.print(actualFanSpeedRpm);
  Serial.println("RPM");
  
  delay(DELAY_TIME);
}
